<?php
/**
 * Linework Customizer Options
 *
 * @package Linework
 */

/**
 * Add theme options to the customizer.
 *
 * @param WP_Customize_Manager $wp_customize Customizer object.
 */
function linework_customize_register( $wp_customize ) {

	// postMessage support for blog name and description.
	$wp_customize->get_setting( 'blogname' )->transport        = 'postMessage';
	$wp_customize->get_setting( 'blogdescription' )->transport = 'postMessage';

	// Add the footer/copyright information section, settings & controls.
	$wp_customize->add_section( 'footer' , array(
		'title'	   => __( 'Footer', 'linework' ),
		'priority' => 115,
	) );

	$wp_customize->add_setting( 'copy_name' , array(
		'default'           => get_bloginfo( 'name' ),
		'transport'         => 'postMessage',
		'sanitize_callback' => 'sanitize_text_field',
	) );

	$wp_customize->add_setting( 'powered_by_wp' , array(
		'default'           => true,
		'transport'         => 'postMessage',
		'sanitize_callback' => 'linework_sanitize_checkbox',
	) );

	$wp_customize->add_setting( 'theme_meta' , array(
		'default'           => false,
		'transport'         => 'postMessage',
		'sanitize_callback' => 'linework_sanitize_checkbox'
	) );

	$wp_customize->add_control( 'copy_name', array(
		'label'		=> __( 'Copyright Name', 'linework' ),
		'section'	=> 'footer',
		'type'      => 'text',
	) );

	$wp_customize->add_control( 'powered_by_wp', array(
		'label'		=> __( 'Proudly Powered By WordPress', 'linework' ),
		'section'	=> 'footer',
		'type'		=> 'checkbox',
	) );

	$wp_customize->add_control( 'theme_meta', array(
		'label'		=> __( 'Theme Information', 'linework' ),
		'section'	=> 'footer',
		'type'		=> 'checkbox',
	) );

	// Add the Linework section.
	$wp_customize->add_section( 'linework' , array(
		'title'	      => __( 'Linework', 'linework' ),
		'description' => __( 'Settings for the linework theme, customizing the linework background and other design elements.', 'linework' ),
		'priority'    => 60,
	) );

	// Add the Linework settings, which all support postMessage.
	$wp_customize->add_setting( 'lw_animate' , array(
		'default'			=> false,
		'transport'         => 'postMessage',
		'sanitize_callback' => 'linework_sanitize_checkbox',
	) );

	$wp_customize->add_setting( 'lw_animate_page' , array(
		'default'			=> false,
		'transport'         => 'postMessage',
		'sanitize_callback' => 'linework_sanitize_checkbox',
	) );

	$wp_customize->add_setting( 'lw_initial' , array(
		'default'			=> 100,
		'transport'         => 'postMessage',
		'sanitize_callback'	=> 'absint',
	) );

	// Add the color settings.
	$wp_customize->add_setting( 'linework_color' , array(
		'default'           => '#20a',
		'transport'         => 'postMessage',
		'sanitize_callback'	=> 'sanitize_hex_color',
//		'validate_callback' => 'linework_validate_colors', @todo enable once notifications can be sent without the `error` state.
	) );

	$wp_customize->add_setting( 'lw_background_color' , array(
		'default'           => '#fffcf7',
		'transport'         => 'postMessage',
		'sanitize_callback'	=> 'sanitize_hex_color',
	) );

	// Add the color controls.
	$wp_customize->add_control( new WP_Customize_Color_Control( $wp_customize, 'linework_color', array(
		'label'        => __( 'Linework Color', 'linework' ),
		'section'    => 'linework',
	) ) );

	$wp_customize->add_control( new WP_Customize_Color_Control( $wp_customize, 'lw_background_color', array(
		'label'        => __( 'Background Color', 'linework' ),
		'section'    => 'linework',
	) ) );

	// Add the Linework controls.
	$wp_customize->add_control( 'lw_initial', array(
		'label'		=> __( 'Initial Line Density', 'linework' ),
		'section'	=> 'linework',
		'type'      => 'range',
		'input_attrs' => array(
			'min'  => 0,
			'max'  => 300,
			'step' => 5,
		),
	) );

	$wp_customize->add_control( 'lw_animate', array(
		'label'		  => __( 'Animate Linework', 'linework' ),
		'section'	  => 'linework',
		'type'        => 'checkbox',
		'description' => __( 'Animate the linework background graphic, drawing new lines over time.', 'linework' ),
	) );

	$wp_customize->add_control( 'lw_animate_page', array(
		'label'		  => __( 'Fade in Page', 'linework' ),
		'section'	  => 'linework',
		'type'        => 'checkbox',
		'description' => __( 'Showcase the linework graphic by fading in the page contents with a brief delay after a pageload.', 'linework' ),
	) );

	// This filter allows the colors CSS to be generated from each of the separate settings and updated as needed.
	add_filter( 'theme_mod_linework_colors_css', 'linework_generate_colors_css' );

	// Partial refresh for better user experience (faster loading of changes).
	// This is a supplement to the initial postMessage setting update that handles PHP 
	// logic more complex than basic color swaps in the CSS (such as contrast ratios).
	$wp_customize->selective_refresh->add_partial( 'linework_colors', array(
		'selector'        => '#linework-colors',
		'settings'        => array( 'linework_color', 'lw_background_color' ),
		'render_callback' => 'linework_generate_colors_css',
	) );

	// Add selective refresh support for the title and tagline, and the footer options.
	$wp_customize->selective_refresh->add_partial( 'blogname', array(
	    'selector' => '.site-title a',
	    'render_callback' => 'linework_customize_partial_blogname',
	) );
	$wp_customize->selective_refresh->add_partial( 'blogdescription', array(
	    'selector' => '.site-description',
	    'render_callback' => 'linework_customize_partial_blogdescription',
	) );
	$wp_customize->selective_refresh->add_partial( 'footer_credits', array(
	    'selector' => '.site-info',
		'settings' => array( 'copy_name', 'powered_by_wp', 'theme_meta' ),
	    'render_callback' => 'linework_site_credits',
	) );

}
add_action( 'customize_register', 'linework_customize_register' );

// Contains functions that generate CSS for all color modification patterns.
require( 'color-patterns.php' );

/**
 * Returns the CSS output of the custom colors.
 *
 * This is useful when the colors css is large enough that it needs to be broken into multiple functions.
 *
 * @since Linework 1.0
 *
 * @return string
 */
function linework_generate_colors_css() {
	return linework_lw_colors_css();
}

/**
 * Caches the CSS output of the custom colors.
 *
 * @since Linework 1.0
 *
 * @return void
 */
function linework_rebuild_color_patterns() {
	set_theme_mod( 'linework_colors_css', linework_generate_colors_css() );
}
// Allow custom colors to work on child themes by not hardcoding "linework".
$linework_theme = get_stylesheet();
add_action( "update_option_theme_mods_$linework_theme", 'linework_rebuild_color_patterns' );

/**
 * Binds JS handlers to make the Customizer preview reload changes asynchronously.
 *
 * @since Linework 1.0
 *
 * @return void
 */
function linework_customize_preview_js() {
	wp_enqueue_script( 'linework_customizer', get_template_directory_uri() . '/js/customizer.js', array( 'customize-preview' ), '20160716', true );
}
add_action( 'customize_preview_init', 'linework_customize_preview_js' );

/**
 * Output custom colors CSS on the front end.
 *
 * @since Linework 1.0
 *
 * @return void
 */
function linework_custom_colors_head() {
	if ( is_customize_preview() ) {
		$data = ' data-linework_color="' . esc_attr( get_theme_mod( 'linework_color', '#20a' ) ) . '" data-background_color="' . esc_attr( get_theme_mod( 'lw_background_color', '#fffcf7' ) ) . '"';
	} else {
		$data = '';
	}
	echo '<style type="text/css" id="linework-colors"' . $data . '>' .
		esc_html( get_theme_mod( 'linework_colors_css', '' ) ) .
	'</style>';
}
add_action( 'wp_head', 'linework_custom_colors_head' );

/**
 * Filter body classes based on customizer settings.
 *
 * @since Linework 1.0
 *
 * @return void
 */
function linework_customize_body_class( $classes ) {
	if ( get_theme_mod( 'lw_animate_page', false ) ) {
		$classes[] = 'fade-in-page';
	}
	
	// Add support for responsive embeds.
	$classes[] = 'wp-embed-responsive';

	return $classes;
}
add_filter( 'body_class', 'linework_customize_body_class' );

/**
 * Sanitize a checkbox input to 1 or 0.
 *
 * @since Linework 1.0
 *
 * @return void
 */
function linework_sanitize_checkbox( $input ) {
    if ( $input ) {
        return 1;
    } else {
        return '';
    }
}

/**
 * Validate the linework or background color settings to ensure minimum color contrast.
 *
 * @since Linework 1.0
 *
 * @return void
 */
function linework_validate_colors( $validity, $value, $setting ) {
	require_once( 'color-calculations.php' );
	if ( 'linework_color' === $setting->id ) {
		$color2 = get_theme_mod( 'lw_background_color', '#fffcf7' );
	} elseif ( 'lw_background_color' == $setting->id ) {
		$color2 = get_theme_mod( 'linework_color', '#20a' );
	}
	if ( empty( $value ) || ! sanitize_hex_color( $value ) ) {
		$validity->add( 'required', __( 'You must specificy a color.', 'linework' ) );
	} elseif ( 4.5 > linework_contrast_ratio( $value, $color2 ) ) {
		$validity->add( 'required', __( 'There is not enough contrast between your selected colors for readibility. Please adjust one or both colors to increase the contrast; an example of an acceptable combination has been displayed in the preview.', 'linework' ) );
	}
	// @todo use a warning instead of a save-blocking error, allowing users to use the generated colors.
	// Updated string when warning notifications are available:
	// There is not enough contrast between your selected Linework and Background colors for readability. The colors have been automatically adjusted to provide minimum required contrast, but you may want to try adjusting the colors yourself.
	return $validity;
}


/**
 * Render the site title for the selective refresh partial.
 *
 * @since Linework 1.0
 * @see linework_customize_register()
 *
 * @return void
 */
function linework_customize_partial_blogname() {
	bloginfo( 'name' );
}

/**
 * Render the site tagline for the selective refresh partial.
 *
 * @since Linework 1.0
 * @see linework_customize_register()
 *
 * @return void
 */
function linework_customize_partial_blogdescription() {
	bloginfo( 'description' );
}
