/**
 * Linework animation script. Runs on a full-window <canvas> element.
 */

var lw = {};
( function( $ ) {

	// Load settings passed from wp_localize_script().
	lw.settings = lineworkSettings;

	// Start the animation.
	lw.init = function() {
		clearInterval( lw.t ); // Allows re-initing via this same function.

		lw.canvas = document.getElementById( 'linework' );
		if ( null === lw.canvas ) {
			return;
		}
		lw.ctx = lw.canvas.getContext('2d');

		lw.resizeCanvas();

		lw.ctx.strokeStyle = lw.settings.color;
		lw.ctx.lineWidth = 1;
		lw.i = 0;

		lw.settings.initLines = Math.ceil( lw.settings.initial * lw.iheight / 300 );
	
		lw.initLines( lw.settings.initLines );
		if ( lw.settings.animate ) {
			lw.t = setInterval( lw.drawLine, 678 );
			$( '#linework-animation-toggle' ).show();
		} else {
			$( '#linework-animation-toggle' ).hide();
		}
	};

	// Run the initial iterations of the animation.
	lw.initLines = function( iterations ) {
		var ii = 0;
		while ( ii < iterations ) {
			lw.drawLine();
			ii++;
		}
	};

	// Alternately draw vertical or horizontal randomly-angled "infinite" lines.
	lw.drawLine = function() {
		var x1 = Math.ceil( Math.random() * lw.iwidth / 8 ) * 8,
		    x2 = Math.ceil( Math.random() * lw.iwidth / 8 ) * 8,
			y1 = Math.ceil( Math.random() * lw.iheight / 8 ) * 8,
			y2 = Math.ceil( Math.random() * lw.iheight / 8 ) * 8;

		lw.ctx.beginPath();
		if ( lw.isEven( lw.i ) ) {
			lw.ctx.moveTo( 0, y1 );
			lw.ctx.lineTo( lw.iwidth - 1, y2 );
		} else {
			lw.ctx.moveTo( x1, 0 );
			lw.ctx.lineTo( x2, lw.iheight - 1 );
		}
		lw.ctx.stroke();
		lw.i++;
	}

	// Returns boolean whether the value is true or false.
	lw.isEven = function( value ) {
		return ( 0 == value % 2 ) ? true : false;
	}

	// Resive the <canvas> element to fill the window, and update the corresponding internal vars. 
	lw.resizeCanvas = function() {
		lw.iwidth = window.innerWidth;
		lw.iheight = window.innerHeight;
		lw.canvas.width = lw.iwidth;
		lw.canvas.height = lw.iheight;
	}

	// Clear the entire canvas, resize as needed, and redraw.
	lw.clearCanvas = function() {
		lw.resizeCanvas(); // Might as well re-set it now if it changed.
		lw.ctx.clearRect( 0, 0, lw.iwidth, lw.iheight );
		lw.ctx.strokeStyle = lw.settings.color;
		lw.initLines( lw.settings.initLines ); // The interval for animating should continue operating.
	}

	$(document).ready( function() {
		lw.init();

		$( window ).resize( function() {
			lw.clearCanvas(); // Must be cleared when size is changed.
		});

		// This works around a browser bug where the canvas isn't displayed after a full refresh in the customizer.
		setTimeout( function() {
			if ( null === lw.canvas ) {
				return;
			}
			lw.canvas.setAttribute( 'style', 'z-index: 1' );
			lw.canvas.setAttribute( 'style', 'z-index: -1' );
		}, 200 );

		$( '#linework-animation-toggle' ).on( 'click', function( e ) {
			if ( $( e.currentTarget ).hasClass( 'on' ) ) {
				$( e.currentTarget ).removeClass( 'on' );
				clearInterval( lw.t );
				clearTimeout( lw.t );
			} else {
				$( e.currentTarget ).addClass( 'on' );
				lw.t = setInterval( lw.drawLine, 678 );
			}
		});
	});

} )( jQuery );